from django.contrib import admin
from arrogant.models import *
# Register your models here.
admin.site.register(Company)
admin.site.register(Job)
admin.site.register(JobTag)
admin.site.register(Category)
admin.site.register(SkillTag)
admin.site.register(Comment)
admin.site.register(LikesFromUser)
admin.site.register(PageLog)


from django.http import JsonResponse, Http404
from django.shortcuts import render
from djangoApiDec.djangoApiDec import queryString_required
from django.forms.models import model_to_dict
from arrogant.models import *
from django.views import View
from django.core import serializers
import json, os, itertools, requests
from infernoWeb.view.inferno import user_verify
from django.db.models import F

school2loc = json.load(open(os.path.join(os.path.dirname(os.path.abspath(__file__)),'school2location.json'), 'r'))
dept2job = json.load(open(os.path.join(os.path.dirname(os.path.abspath(__file__)),'dept2job.json'), 'r'))
AMOUNT_NUM = 10
SEARCH_NUM = 5

@queryString_required(['dept'])
def recommendJvalue(request):
    import random
    dept = request.GET['dept']
    for i in dept2job:
        if i in dept:
            random.shuffle(dept2job[i])
            jid = Category.objects.get(name=dept2job[i][0]).job_set.random().id
            request.GET = request.GET.copy()
            request.GET['id'] = jid
            return jvalue(request)
    return JsonResponse({}, safe=False)

@queryString_required(['id'])
def jvalue(request):
    j = Job.objects.prefetch_related('jobtag_set', 'category', 'skilltag_set', 'company').get(id=request.GET['id'])
    result = model_to_dict(j, exclude='attendee')
    result['company'] = j.company.natural_key()
    result['JobTag'] = list(j.jobtag_set.all().values())
    result['skilltag'] = list(j.skilltag_set.all().values())
    result['Category'] = j.category.name
    return JsonResponse(result, safe=False)

@queryString_required(['start'])
def jlist(request):
    start = int(request.GET['start']) - 1
    category = request.GET['category'] if 'category' in request.GET else "行銷/社群經營"
    querySet = Category.objects.get(name=category).job_set.select_related('company').prefetch_related('jobtag_set', 'skilltag_set').all()
    length = len(querySet) // AMOUNT_NUM +1

    result = []
    querySet = querySet[start:start+AMOUNT_NUM]
    for i in querySet:
        tmp = model_to_dict(i, exclude='attendee')
        tmp['company'] = i.company.natural_key()
        tmp['jobtag'] = [tag.name for tag in i.jobtag_set.all()]
        tmp['skilltag'] = [(tag.name, tag.skill_field) for tag in i.skilltag_set.all()]
        result.append(tmp)

    return JsonResponse([{'TotalPage':length, 'category':category}] + result, safe=False)

def jcategory(request):
    return JsonResponse(json.loads(serializers.serialize('json', Category.objects.all(), fields=('name'))), safe=False)

# 顯示特定一門課程的留言評論
@queryString_required(['id', 'start'])
def comment(request):
    try:
        start = int(request.GET['start']) - 1
        j = Job.objects.prefetch_related('comment_set').get(id=request.GET['id'])
        comments = j.comment_set.all()[start:start+AMOUNT_NUM]

        result = []
        for i, j in zip(json.loads(serializers.serialize('json', comments, use_natural_foreign_keys=True, use_natural_primary_keys=True)), comments):
            i['fields']['likesfromuser'] = list(map(lambda x:x.author.facebookid, j.likesfromuser_set.all()))
            result.append(i)

        return JsonResponse(result, safe=False)
    except Exception as e:
        raise


# 建立特定一門課程的留言評論
@queryString_required(['id'])
@user_verify
def CreateComment(request):
    id = request.GET['id']
    j = Job.objects.prefetch_related('comment_set').get(id=id)
    if len(j.comment_set.all().filter(author=User.objects.get(facebookid=request.POST['id'])))==0:
        Comment.objects.create(Job=j, author=User.objects.get(facebookid=request.POST['id']) , create=timezone.now(), raw=request.POST['comments'], emotion=request.POST['emotion'])
        return True
    return False

def logPage(request):
    PageLog.objects.create(user=User.objects.get(facebookid=request.POST['id']), Job=Job.objects.get(id=request.GET['id']), create=timezone.now())

@queryString_required(['id'])
@user_verify
def like(request):
    request.GET = request.GET.copy()
    request.GET['start'] = 1
    if request.POST:
        user = User.objects.get(facebookid=request.POST['id'])
        if request.POST['like'] == '1':
            target = Comment.objects.filter(id=request.GET['id'])
            target.update(like=F('like') + int(request.POST['like']))
            obj, created = LikesFromUser.objects.get_or_create(author=user)
            obj.comment.add(target[0])
            return JsonResponse({"like":'success'})
        elif request.POST['like'] == '-1':
            Comment.objects.filter(id=request.GET['id']).update(like=F('like') + int(request.POST['like']))
            LikesFromUser.objects.get(author=user).comment.remove(Comment.objects.get(id=request.GET['id']))
            return JsonResponse({"like":'success'})

@user_verify
@queryString_required(['id'])
def questionnaire(request):
    id = request.GET['id']
    j = Job.objects.get(id=id)
    if request.method == 'POST' and request.POST:
        if User.objects.get(facebookid=request.POST['id']) in j.attendee.all():
            return JsonResponse({'alreadySubmit':True})
        if 'rating' in request.POST:
            data = json.loads(request.POST['rating'])
            amount = j.feedback_amount + 1
            modelDict = {'feedback_amount':amount}
            modelDict['feedback_freedom'] = (j.feedback_freedom*(amount-1) + (data[0]*3/4 + data[1]/4)) /amount
            modelDict['feedback_salary'] = (j.feedback_salary*(amount-1) + data[2]) / amount
            modelDict['feedback_easy'] = (j.feedback_easy*(amount-1) + (data[3]/12 + data[4]/12  + data[7]*9/12 + data[8]/12)) / amount
            modelDict['feedback_knowledgeable'] = (j.feedback_knowledgeable*(amount-1) + data[6]) / amount
            modelDict['feedback_FU'] = (j.feedback_FU*(amount-1) + data[5]) / amount
            Job.objects.update_or_create(id=id, defaults=modelDict)
            Job.objects.get(id=id).attendee.add(User.objects.get(facebookid=request.POST['id']))
            return JsonResponse({'submitSuccess':True})

@queryString_required(['keyword'])
def search(request):
    querySet = Job.objects.filter(name__contains=request.GET['keyword']).select_related('company').prefetch_related('jobtag_set', 'skilltag_set', 'company')[:SEARCH_NUM]
    if querySet.count() == 0:
        nlpapi_Expand =  list(itertools.chain(
            *itertools.chain(*zip(
                requests.get('http://140.120.13.244:10000/kem/?keyword={}&lang=cht'.format(request.GET['keyword'])).json(), 
                requests.get('http://140.120.13.244:10000/kcm/?keyword={}&lang=cht'.format(request.GET['keyword'])).json())
            )
        ))[::2]
        nlpapiList = []
        for i in nlpapi_Expand:
            querySet = Job.objects.filter(name__contains=i).select_related('company').prefetch_related('jobtag_set', 'skilltag_set', 'company')[:SEARCH_NUM]
    result = []
    for i in querySet:
        tmp = model_to_dict(i, exclude='attendee')
        tmp['company'] = i.company.natural_key()
        tmp['jobtag'] = [tag.name for tag in i.jobtag_set.all()]
        tmp['skilltag'] = [(tag.name, tag.skill_field) for tag in i.skilltag_set.all()]
        result.append(tmp)
    return JsonResponse(result, safe=False)


from django.core.management.base import BaseCommand, CommandError
from arrogant.models import *
import json, pyprind

class Command(BaseCommand):
    help = 'Convenient Way to insert Intern of Yourator json into arrogant'
    def add_arguments(self, parser):
            # Positional arguments
            parser.add_argument('json', type=str)

    def handle(self, *args, **options):
        file = options['json']
        with open(file, 'r') as f:
            for i in pyprind.prog_bar(json.load(f)):
                company = self.getOrCreateCompany(i)
                category = self.getOrCreateCategory(i['category'])
                job = self.getOrCreateJob(company, category, i)
                tags = self.getOrCreateTag(i['tags'], job)
                SkillTag = self.getOrCreateSkillTag(i['skill_tags'], job)

        self.stdout.write(self.style.SUCCESS('insert Json success!!!'))

    @staticmethod
    def getOrCreateCompany(i):
        obj, _ = Company.objects.get_or_create(
            brand=i['company']['brand'],
            banner=i['company']['banner'],
            path=i['company']['path'],
            area=i['company']['area'],
            公司規模=i['inside'].get('公司規模', '未公開'),
            地址=i['inside']['地址'],
            資本額=i['inside'].get('資本額', "未公開"),
            description=i['inside']['description']
        )
        return obj

    @staticmethod
    def getOrCreateJob(company, category, i):
        obj, _ = Job.objects.get_or_create(
            name=i['name'],
            intern_tf=i['intern'],
            has_salary_info=i['has_salary_info'],
            salary=i.get('salary', "未公開"),
            path=i['path'],
            avatar=i['company']['banner'],
            company=company,
            category=category
        )
        return obj

    @staticmethod
    def getOrCreateTag(tags, job):
        result = []
        for i in tags:
            obj, _ = JobTag.objects.get_or_create(
                name=i['name'],
            )
            obj.Job.add(job)
            result.append(obj)
        return result

    @staticmethod
    def getOrCreateCategory(i):
        obj, _ = Category.objects.get_or_create(
            name=i['name'],
        )
        return obj

    @staticmethod
    def getOrCreateSkillTag(skills, job):
        result = []
        for i in skills:
            obj, _ = SkillTag.objects.get_or_create(
                name=i['name'],
                skill_field=i['skill_field'],
            )
            obj.Job.add(job)
            result.append(obj)
        return result



from django.db import models
from django.utils import timezone
from infernoWeb.models import User
from django.db.models import Count
from random import randint

# Create your models here.
class CompanyManager(models.Manager):
    def get_by_natural_key(self, brand):
        return self.get(brand=brand)

class Company(models.Model):
    objects = CompanyManager()

    brand = models.CharField(max_length=30, default='')
    banner = models.CharField(max_length=200, default='') # 大頭貼照片
    path = models.CharField(max_length=40, default='')
    area = models.CharField(max_length=10, default='')
    公司規模 = models.CharField(max_length=5, default='未公開')
    地址 = models.CharField(max_length=40, default='')
    資本額 = models.CharField(max_length=10, default='未公開')
    description = models.CharField(max_length=150, default='')
    def __str__(self):
        return self.brand

    def natural_key(self):
        return {
            "brand":self.brand,
            "banner":self.banner,
            "area":self.area,
            "path": self.path,
            "公司規模": self.公司規模,
            "地址": self.地址,
            "資本額": self.資本額,
            "description": self.description
        }

    class Meta:
        unique_together = ('brand', 'path')
        
class Category(models.Model):
    name = models.CharField(max_length=15)
    def __str__(self):
        return self.name
        
class JobManager(models.Manager):
    def random(self):
        count = self.aggregate(ids=Count('id'))['ids']
        random_index = randint(0, count - 1)
        return self.all()[random_index]

class Job(models.Model):
    """docstring for Job"""
    objects = JobManager()
    
    category = models.ForeignKey(Category)
    company = models.ForeignKey(Company)
    name = models.CharField(max_length=20, default='')
    intern_tf = models.BooleanField(default=False)
    has_salary_info = models.BooleanField(default=False)
    salary = models.CharField(max_length=30, default='未公開')
    path = models.CharField(max_length=40, default='')
    avatar = models.CharField(max_length=200, default='') # 大頭貼照片
    feedback_amount = models.PositiveIntegerField(default=0)
    feedback_freedom = models.FloatField(default=3)
    feedback_FU = models.FloatField(default=3)
    feedback_easy = models.FloatField(default=3)
    feedback_salary = models.FloatField(default=3)
    feedback_knowledgeable = models.FloatField(default=3)
    attendee = models.ManyToManyField(User)
    def __str__(self):
        return self.name

class JobTag(models.Model):
    name = models.CharField(max_length=15)
    Job = models.ManyToManyField(Job)
    def __str__(self):
        return self.name



class SkillTag(models.Model):
    name = models.CharField(max_length=15)
    skill_field = models.CharField(max_length=10)
    Job = models.ManyToManyField(Job)
    def __str__(self):
        return self.name + '-' + '-' + self.skill_field



class Comment(models.Model):
    Job = models.ForeignKey(Job)
    author = models.ForeignKey(User, related_name='cmtauthor', default=10)
    create = models.DateTimeField(default=timezone.now)
    raw = models.CharField(max_length=500)
    like = models.PositiveSmallIntegerField(default=0)
    emotion = models.CharField(
        max_length=7,
        choices=(
            ('neutral', 'neutral'),
            ('pos', 'pos'),
            ('neg', 'neg'),
        ),
        default='neutral',
    )
    def __str__(self):
        return self.raw

class LikesFromUser(models.Model):
    author = models.OneToOneField(User, related_name='lfuauthor')
    comment = models.ManyToManyField(Comment)   
    def __str__(self):
        return self.author.name

class PageLog(models.Model):
    user = models.ForeignKey(User, related_name='ploguser')
    Job = models.ForeignKey(Job)
    create = models.DateTimeField(default=timezone.now)
    def __str__(self):
        return self.user.name + self.Job.name + self.create.date().__str__()


# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-26 02:34
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('infernoWeb', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=15)),
            ],
        ),
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('create', models.DateTimeField(default=django.utils.timezone.now)),
                ('raw', models.CharField(max_length=500)),
                ('like', models.PositiveSmallIntegerField(default=0)),
                ('emotion', models.CharField(choices=[('neutral', 'neutral'), ('pos', 'pos'), ('neg', 'neg')], default='neutral', max_length=7)),
            ],
        ),
        migrations.CreateModel(
            name='Company',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('brand', models.CharField(default='', max_length=30)),
                ('banner', models.CharField(default='', max_length=200)),
                ('path', models.CharField(default='', max_length=40)),
                ('area', models.CharField(default='', max_length=10)),
                ('公司規模', models.CharField(default='未公開', max_length=5)),
                ('地址', models.CharField(default='', max_length=40)),
                ('資本額', models.CharField(default='未公開', max_length=10)),
                ('description', models.CharField(default='', max_length=150)),
            ],
        ),
        migrations.CreateModel(
            name='Job',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(default='', max_length=20)),
                ('intern_tf', models.BooleanField(default=False)),
                ('has_salary_info', models.BooleanField(default=False)),
                ('salary', models.CharField(default='未公開', max_length=30)),
                ('path', models.CharField(default='', max_length=40)),
                ('avatar', models.CharField(default='', max_length=200)),
                ('feedback_amount', models.PositiveIntegerField(default=0)),
                ('feedback_freedom', models.FloatField(default=3)),
                ('feedback_FU', models.FloatField(default=3)),
                ('feedback_easy', models.FloatField(default=3)),
                ('feedback_salary', models.FloatField(default=3)),
                ('feedback_knowledgeable', models.FloatField(default=3)),
                ('attendee', models.ManyToManyField(to='infernoWeb.User')),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='arrogant.Category')),
                ('company', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='arrogant.Company')),
            ],
        ),
        migrations.CreateModel(
            name='JobTag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=15)),
                ('Job', models.ManyToManyField(to='arrogant.Job')),
            ],
        ),
        migrations.CreateModel(
            name='LikesFromUser',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('author', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='lfuauthor', to='infernoWeb.User')),
                ('comment', models.ManyToManyField(to='arrogant.Comment')),
            ],
        ),
        migrations.CreateModel(
            name='PageLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('create', models.DateTimeField(default=django.utils.timezone.now)),
                ('Job', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='arrogant.Job')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='ploguser', to='infernoWeb.User')),
            ],
        ),
        migrations.CreateModel(
            name='SkillTag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=15)),
                ('skill_field', models.CharField(max_length=10)),
                ('Job', models.ManyToManyField(to='arrogant.Job')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='company',
            unique_together=set([('brand', 'path')]),
        ),
        migrations.AddField(
            model_name='comment',
            name='Job',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='arrogant.Job'),
        ),
        migrations.AddField(
            model_name='comment',
            name='author',
            field=models.ForeignKey(default=10, on_delete=django.db.models.deletion.CASCADE, related_name='cmtauthor', to='infernoWeb.User'),
        ),
    ]
